@@ -42,7 +42,7 @@
   :type 'boolean
   :group 'epa)
 
-(defcustom epa-popup-info-window nil
+(defcustom epa-popup-info-window t
   "If non-nil, status information from epa commands is displayed on
 the separate window."
   :type 'boolean
@@ -174,7 +174,6 @@ the separate window."
     (define-key keymap "d" 'epa-decrypt-file)
     (define-key keymap "v" 'epa-verify-file)
     (define-key keymap "s" 'epa-sign-file)
-    (define-key keymap "S" 'epa-sign-keys)
     (define-key keymap "e" 'epa-encrypt-file)
     (define-key keymap "r" 'epa-delete-keys)
     (define-key keymap "i" 'epa-import-keys)
@@ -221,9 +220,11 @@ the separate window."
 		      ? ))
 	    (epg-sub-key-id primary-sub-key)
 	    " "
-	    (if (stringp (epg-user-id-string primary-user-id))
-		(epg-user-id-string primary-user-id)
-	      (epg-decode-dn (epg-user-id-string primary-user-id))))))
+	    (if primary-user-id
+		(if (stringp (epg-user-id-string primary-user-id))
+		    (epg-user-id-string primary-user-id)
+		  (epg-decode-dn (epg-user-id-string primary-user-id)))
+	      ""))))
 
 (defun epa--key-widget-button-face-get (widget)
   (let ((validity (epg-sub-key-validity (car (epg-key-sub-key-list
@@ -246,13 +247,11 @@ the separate window."
 	truncate-lines t
 	buffer-read-only t)
   (use-local-map epa-keys-mode-map)
-  (set-keymap-parent (current-local-map) widget-keymap)
   (make-local-variable 'font-lock-defaults)
   (setq font-lock-defaults '(epa-font-lock-keywords t))
   ;; In XEmacs, auto-initialization of font-lock is not effective
   ;; if buffer-file-name is not set.
   (font-lock-set-defaults)
-  (widget-setup)
   (make-local-variable 'epa-exit-buffer-function)
   (run-hooks 'epa-keys-mode-hook))
 
@@ -344,7 +343,9 @@ If ARG is non-nil, mark the current line."
 			 (point-max)))
       (goto-char point))
     (epa-insert-keys context name mode)
-    (epa-keys-mode))
+    (epa-keys-mode)
+    (widget-setup)
+    (set-keymap-parent (current-local-map) widget-keymap))
   (make-local-variable 'epa-list-keys-arguments)
   (setq epa-list-keys-arguments (list name mode protocol))
   (goto-char (point-min))
@@ -435,6 +436,8 @@ If SECRET is non-nil, list secret keys instead of public keys."
 		  (epa-mark)))
 	  (epa-insert-keys context nil nil)))
       (epa-keys-mode)
+      (widget-setup)
+      (set-keymap-parent (current-local-map) widget-keymap)
       (setq epa-exit-buffer-function #'abort-recursive-edit)
       (goto-char (point-min))
       (pop-to-buffer (current-buffer)))
@@ -466,17 +469,18 @@ If SECRET is non-nil, list secret keys instead of public keys."
     (erase-buffer)
     (setq pointer (epg-key-user-id-list key))
     (while pointer
-      (insert " "
-	      (if (epg-user-id-validity (car pointer))
-		  (char-to-string
-		   (car (rassq (epg-user-id-validity (car pointer))
-			       epg-key-validity-alist)))
-		" ")
-	      " "
-	      (if (stringp (epg-user-id-string (car pointer)))
-		  (epg-user-id-string (car pointer))
-		(epg-decode-dn (epg-user-id-string (car pointer))))
-	      "\n")
+      (if (car pointer)
+	  (insert " "
+		  (if (epg-user-id-validity (car pointer))
+		      (char-to-string
+		       (car (rassq (epg-user-id-validity (car pointer))
+				   epg-key-validity-alist)))
+		    " ")
+		  " "
+		  (if (stringp (epg-user-id-string (car pointer)))
+		      (epg-user-id-string (car pointer))
+		    (epg-decode-dn (epg-user-id-string (car pointer))))
+		  "\n"))
       (setq pointer (cdr pointer)))
     (setq pointer (epg-key-sub-key-list key))
     (while pointer
@@ -515,9 +519,9 @@ If SECRET is non-nil, list secret keys instead of public keys."
     (pop-to-buffer (current-buffer))
     (epa-key-mode)))
 
-(defun epa-display-verify-result (verify-result)
+(defun epa-display-info (info)
   (if epa-popup-info-window
-      (progn
+      (save-selected-window
 	(unless epa-info-buffer
 	  (setq epa-info-buffer (generate-new-buffer "*Info*")))
 	(save-excursion
@@ -525,13 +529,22 @@ If SECRET is non-nil, list secret keys instead of public keys."
 	  (let ((inhibit-read-only t)
 		buffer-read-only)
 	    (erase-buffer)
-	    (insert (epg-verify-result-to-string verify-result)))
-	  (epa-info-mode))
-	(pop-to-buffer epa-info-buffer)
-	(if (> (window-height) epa-info-window-height)
-	    (shrink-window (- (window-height) epa-info-window-height)))
-	(goto-char (point-min)))
-    (message "%s" (epg-verify-result-to-string verify-result))))
+	    (insert info))
+	  (epa-info-mode)
+	  (goto-char (point-min)))
+	(if (> (window-height)
+	       epa-info-window-height)
+	    (set-window-buffer (split-window nil (- (window-height)
+						    epa-info-window-height))
+			       epa-info-buffer)
+	  (pop-to-buffer epa-info-buffer)
+	  (if (> (window-height) epa-info-window-height)
+	      (shrink-window (- (window-height) epa-info-window-height)))))
+    (message "%s" info)))
+
+(defun epa-display-verify-result (verify-result)
+  (epa-display-info (epg-verify-result-to-string verify-result)))
+(make-obsolete 'epa-display-verify-result 'epa-display-info)
 
 (defun epa-passphrase-callback-function (context key-id handback)
   (if (eq key-id 'SYM)
@@ -555,6 +568,7 @@ If SECRET is non-nil, list secret keys instead of public keys."
 (defun epa-decrypt-file (file)
   "Decrypt FILE."
   (interactive "fFile: ")
+  (setq file (expand-file-name file))
   (let* ((default-name (file-name-sans-extension file))
 	 (plain (expand-file-name
 		 (read-file-name
@@ -570,14 +584,17 @@ If SECRET is non-nil, list secret keys instead of public keys."
 				       #'epa-progress-callback-function)
     (message "Decrypting %s..." (file-name-nondirectory file))
     (epg-decrypt-file context file plain)
-    (message "Decrypting %s...done" (file-name-nondirectory file))
+    (message "Decrypting %s...wrote %s" (file-name-nondirectory file)
+	     (file-name-nondirectory plain))
     (if (epg-context-result-for context 'verify)
-	(epa-display-verify-result (epg-context-result-for context 'verify)))))
+	(epa-display-info (epg-verify-result-to-string
+			   (epg-context-result-for context 'verify))))))
 
 ;;;###autoload
 (defun epa-verify-file (file)
   "Verify FILE."
   (interactive "fFile: ")
+  (setq file (expand-file-name file))
   (let* ((context (epg-make-context))
 	 (plain (if (equal (file-name-extension file) "sig")
 		    (file-name-sans-extension file))))
@@ -587,7 +604,8 @@ If SECRET is non-nil, list secret keys instead of public keys."
     (epg-verify-file context file plain)
     (message "Verifying %s...done" (file-name-nondirectory file))
     (if (epg-context-result-for context 'verify)
-	(epa-display-verify-result (epg-context-result-for context 'verify)))))
+	(epa-display-info (epg-verify-result-to-string
+			   (epg-context-result-for context 'verify))))))
 
 ;;;###autoload
 (defun epa-sign-file (file signers mode)
@@ -597,10 +615,26 @@ If SECRET is non-nil, list secret keys instead of public keys."
 	 (epa-select-keys (epg-make-context) "Select keys for signing.
 If no one is selected, default secret key is used.  "
 			  nil t)
-	 (if (y-or-n-p "Make a detached signature? ")
-	     'detached
-	   (if (y-or-n-p "Make a cleartext signature? ")
-	       'clear))))
+	 (catch 'done
+	   (while t
+	     (message "Signature type (n,c,d,?) ")
+	     (let ((c (read-char)))
+	       (cond ((eq c ?c)
+		      (throw 'done 'clear))
+		     ((eq c ?d)
+		      (throw 'done 'detached))
+		     ((eq c ??)
+		      (with-output-to-temp-buffer "*Help*"
+			(save-excursion
+			  (set-buffer standard-output)
+			  (insert "\
+n - Create a normal signature
+c - Create a cleartext signature
+d - Create a detached signature
+? - Show this help
+"))))
+		     (t
+		      (throw 'done nil))))))))
   (let ((signature (concat file
 			   (if (or epa-armor
 				   (not (memq mode '(nil t normal detached))))
@@ -618,7 +652,8 @@ If no one is selected, default secret key is used.  "
 				       #'epa-progress-callback-function)
     (message "Signing %s..." (file-name-nondirectory file))
     (epg-sign-file context file signature mode)
-    (message "Signing %s...done" (file-name-nondirectory file))))
+    (message "Signing %s...wrote %s" (file-name-nondirectory file)
+	     (file-name-nondirectory signature))))
 
 ;;;###autoload
 (defun epa-encrypt-file (file recipients)
@@ -637,7 +672,8 @@ If no one is selected, symmetric encryption will be performed.  ")))
 				       #'epa-progress-callback-function)
     (message "Encrypting %s..." (file-name-nondirectory file))
     (epg-encrypt-file context file recipients cipher)
-    (message "Encrypting %s...done" (file-name-nondirectory file))))
+    (message "Encrypting %s...wrote %s" (file-name-nondirectory file)
+	     (file-name-nondirectory cipher))))
 
 ;;;###autoload
 (defun epa-decrypt-region (start end)
@@ -659,7 +695,8 @@ Don't use this command in Lisp programs!"
       (goto-char start)
       (insert (decode-coding-string plain coding-system-for-read))
       (if (epg-context-result-for context 'verify)
-	  (epa-display-verify-result (epg-context-result-for context 'verify))))))
+	  (epa-display-info (epg-verify-result-to-string
+			     (epg-context-result-for context 'verify)))))))
 
 ;;;###autoload
 (defun epa-decrypt-armor-in-region (start end)
@@ -703,7 +740,8 @@ Don't use this command in Lisp programs!"
 			(buffer-substring start end)
 			coding-system-for-write))
     (if (epg-context-result-for context 'verify)
-	(epa-display-verify-result (epg-context-result-for context 'verify)))))
+	(epa-display-info (epg-verify-result-to-string
+			   (epg-context-result-for context 'verify))))))
 
 ;;;###autoload
 (defun epa-verify-cleartext-in-region (start end)
@@ -740,10 +778,26 @@ Don't use this command in Lisp programs!"
 	 (epa-select-keys (epg-make-context) "Select keys for signing.
 If no one is selected, default secret key is used.  "
 			  nil t)
-	 (if (y-or-n-p "Make a detached signature? ")
-	     'detached
-	   (if (y-or-n-p "Make a cleartext signature? ")
-	       'clear))))
+	 (catch 'done
+	   (while t
+	     (message "Signature type (n,c,d,?) ")
+	     (let ((c (read-char)))
+	       (cond ((eq c ?c)
+		      (throw 'done 'clear))
+		     ((eq c ?d)
+		      (throw 'done 'detached))
+		     ((eq c ??)
+		      (with-output-to-temp-buffer "*Help*"
+			(save-excursion
+			  (set-buffer standard-output)
+			  (insert "\
+n - Create a normal signature
+c - Create a cleartext signature
+d - Create a detached signature
+? - Show this help
+"))))
+		     (t
+		      (throw 'done nil))))))))
   (save-excursion
     (let ((context (epg-make-context))
 	  signature)
@@ -815,11 +869,20 @@ Don't use this command in Lisp programs!"
 
 Don't use this command in Lisp programs!"
   (interactive "fFile: ")
+  (setq file (expand-file-name file))
   (let ((context (epg-make-context)))
     (message "Importing %s..." (file-name-nondirectory file))
-    (epg-import-keys-from-file context (expand-file-name file))
-    (message "Importing %s...done" (file-name-nondirectory file))
-    (apply #'epa-list-keys epa-list-keys-arguments)))
+    (condition-case nil
+	(progn
+	  (epg-import-keys-from-file context file)
+	  (message "Importing %s...done" (file-name-nondirectory file)))
+      (error
+       (message "Importing %s...failed" (file-name-nondirectory file))))
+    (if (epg-context-result-for context 'import)
+	(epa-display-info (epg-import-result-to-string
+			   (epg-context-result-for context 'import))))
+    (if (eq major-mode 'epa-keys-mode)
+	(apply #'epa-list-keys epa-list-keys-arguments))))
 
 ;;;###autoload
 (defun epa-export-keys (keys file)
@@ -869,6 +932,7 @@ Don't use this command in Lisp programs!"
     (message "Signing keys...")
     (epg-sign-keys context keys local)
     (message "Signing keys...done")))
+(make-obsolete 'epa-sign-keys "Do not use.")
 
 (provide 'epa)
 
